//
//  WBOViewController.m
//  MicroLifeDeviceSDK-Dome
//
//  Created by willy.wu on 2019/10/2.
//  Copyright © 2019 WiFi@MBP. All rights reserved.
//

#import "WBOViewController.h"

@interface WBOViewController () <WBODataResponseDelegate> {
    Dformat dformat;
    MicroLifeSettingValues *aSettingValues;
}
@property (strong, nonatomic) WBOBLEManager *aWBOBLEManager;
@property (assign, nonatomic) int SWCBPindexMax;

@property (weak, nonatomic) IBOutlet UITextField *editUserID;
@property (weak, nonatomic) IBOutlet UITextField *editCBPindex;

@property (weak, nonatomic) IBOutlet UITextField *editAUS_HI_infPressure;
@property (weak, nonatomic) IBOutlet UITextField *editHI_infPressure;
@property (weak, nonatomic) IBOutlet UISwitch *SW_AUTO_hide;
@property (weak, nonatomic) IBOutlet UISwitch *SW_SEL_silent;
@property (weak, nonatomic) IBOutlet UISwitch *SW_AUS_Hide;
@property (weak, nonatomic) IBOutlet UISwitch *SW_AVG_no_include_first;
@property (weak, nonatomic) IBOutlet UISwitch *SW_CBP;
@property (weak, nonatomic) IBOutlet UISwitch *SW_AFib;
@property (weak, nonatomic) IBOutlet UISwitch *SW_AMPM;
@property (weak, nonatomic) IBOutlet UISwitch *SW_Kpa;
@property (weak, nonatomic) IBOutlet UITextField *editRestTime;
@property (weak, nonatomic) IBOutlet UITextField *editIntervalTime;
@property (weak, nonatomic) IBOutlet UITextField *editAutoMeasureNumber;

@end

@implementation WBOViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.aWBOBLEManager = [WBOBLEManager shareInstanceWhithAuthorizationkey:SDKkey_WBP];
    self.aWBOBLEManager.dataResponseDelegate = self;
    
    self.wboBindingDevice = [self.aWBOBLEManager getBindingDevice];
    
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [self addLogWhitText:@"WBOBLEManager Start"];
    NSString *log = [NSString stringWithFormat:@"WBOBLEManager bpmBindingDevice:%@",self.wboBindingDevice];
    [self addLogWhitText:log];
    [self bindingDeviceWhitMacAddres:self.wboBindingDevice];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    // Disconnect Device
    [self.aWBOBLEManager disconnectDevice];
}

- (void)viewDidLayoutSubviews {
    self.butView.contentSize = CGSizeMake(self.butView.frame.size.width, self.butView.frame.size.height*2);
}

#pragma mark - settingValues
- (void)setSV {
    self.editAUS_HI_infPressure.text = [NSString stringWithFormat:@"%d",aSettingValues.AUS_HI_infPressure];
    self.editHI_infPressure.text = [NSString stringWithFormat:@"%d",aSettingValues.HI_infPressure];
    self.SW_AUTO_hide.on = aSettingValues.SW_AUTO_hide;
    self.SW_SEL_silent.on = aSettingValues.SW_SEL_silent;
    self.SW_AUS_Hide.on = aSettingValues.SW_AUS_Hide;
    self.SW_AVG_no_include_first.on = aSettingValues.SW_AVG_no_include_first;
    self.SW_CBP.on = aSettingValues.SW_CBP;
    self.SW_AFib.on = aSettingValues.SW_AFib;
    self.SW_AMPM.on = aSettingValues.SW_AMPM;
    self.SW_Kpa.on = aSettingValues.SW_Kpa;
    self.editRestTime.text = [NSString stringWithFormat:@"%d",aSettingValues.RestTime];
    self.editIntervalTime.text = [NSString stringWithFormat:@"%d",aSettingValues.IntervalTime];
    self.editAutoMeasureNumber.text = [NSString stringWithFormat:@"%d",aSettingValues.AutoMeasureNumber];
}

- (void)editSV {
    aSettingValues.AUS_HI_infPressure = self.editAUS_HI_infPressure.text.intValue;
    aSettingValues.HI_infPressure = self.editHI_infPressure.text.intValue;
    aSettingValues.SW_AUTO_hide = self.SW_AUTO_hide.on;
    aSettingValues.SW_SEL_silent = self.SW_SEL_silent.on;
    aSettingValues.SW_AUS_Hide = self.SW_AUS_Hide.on;
    aSettingValues.SW_AVG_no_include_first = self.SW_AVG_no_include_first.on;
    aSettingValues.SW_CBP = self.SW_CBP.on;
    aSettingValues.SW_AFib = self.SW_AFib.on;
    aSettingValues.SW_AMPM = self.SW_AMPM.on;
    aSettingValues.SW_Kpa = self.SW_Kpa.on;
    aSettingValues.RestTime = self.editRestTime.text.intValue;
    aSettingValues.IntervalTime = self.editIntervalTime.text.intValue;
    aSettingValues.AutoMeasureNumber = self.editAutoMeasureNumber.text.intValue;
}

#pragma mark - selected info
- (IBAction)editCBPRaw:(UISegmentedControl *)sender {
    NSString *format = @"No CBP Raw";
    switch (sender.selectedSegmentIndex) {
        case 0:
            format = @"No CBP Raw";
            dformat = MicroLifeDNoCBPRaw;
            break;
        case 1:
            format = @"Low CBP Raw";
            dformat = MicroLifeDLowCBPRaw;
            break;
        case 2:
            format = @"Full CBP Raw";
            dformat = MicroLifeDFullCBPRaw;
            break;
        default:
            break;
    }
    NSString *log = [NSString stringWithFormat:@"WBOBLEManager selectedCBPRaw:%@",format];
    [self addLogWhitText:log];
}
- (IBAction)editSettingValues:(UISwitch *)sender {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManager editSettingValues:"];
    switch ([sender tag]) {
        case 200:
            aSettingValues.SW_AUTO_hide = sender.on;
            log = [log stringByAppendingFormat:@"SW_AUTO_hide"];
            break;
        case 201:
            aSettingValues.SW_SEL_silent = sender.on;
            log = [log stringByAppendingFormat:@"SW_SEL_silent"];
            break;
        case 202:
            aSettingValues.SW_AUS_Hide = sender.on;
            log = [log stringByAppendingFormat:@"SW_AUS_Hide"];
            break;
        case 203:
            aSettingValues.SW_AVG_no_include_first = sender.on;
            log = [log stringByAppendingFormat:@"SW_AVG_no_include_first"];
            break;
        case 204:
            aSettingValues.SW_CBP = sender.on;
            log = [log stringByAppendingFormat:@"SW_CBP"];
            break;
        case 205:
            aSettingValues.SW_AFib = sender.on;
            log = [log stringByAppendingFormat:@"SW_AFib"];
            break;
        case 206:
            aSettingValues.SW_AMPM = sender.on;
            log = [log stringByAppendingFormat:@"SW_AMPM"];
            break;
        case 207:
            aSettingValues.SW_Kpa = sender.on;
            log = [log stringByAppendingFormat:@"SW_Kpa"];
            break;
        default:
            break;
    }
    log = [log stringByAppendingFormat:@" %@",sender.on?@"YES":@"NO"];
    [self addLogWhitText:log];
}

#pragma mark - IBAction
- (IBAction)getBindingDevice:(id)sender {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManager getBindingDevice:%@",[self.aWBOBLEManager getBindingDevice]];
    [self addLogWhitText:log];
}
- (IBAction)bindingDevice:(id)sender {
    [self.aWBOBLEManager bindingDevice:self.wboBindingDevice];
    NSString *log = [NSString stringWithFormat:@"WBOBLEManager bindingDevice:%@",self.wboBindingDevice];
    [self addLogWhitText:log];
    [self bindingDeviceWhitMacAddres:self.wboBindingDevice];
}
- (IBAction)unBindingDevice:(id)sender {
    [self.aWBOBLEManager unBindingDevice];
    NSString *log = [NSString stringWithFormat:@"WBOBLEManager unBindingDevice:%@",[self.aWBOBLEManager getBindingDevice]];
    [self addLogWhitText:log];
}
- (IBAction)reScan:(id)sender {
    [self.aWBOBLEManager reScan];
    [self addLogWhitText:@"WBOBLEManager reScan"];
}
- (IBAction)stopScan:(id)sender {
    [self.aWBOBLEManager stopScan];
    [self addLogWhitText:@"WBOBLEManager stopScan"];
}
- (IBAction)disconnectDevice:(id)sender {
    [self.aWBOBLEManager disconnectDevice];
    [self addLogWhitText:@"WBOBLEManager disconnectDevice"];
}
- (IBAction)comAction:(UIButton *)sender {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManager  %@",sender.titleLabel.text];
    switch ([sender tag]) {
        case 100:
            [self.aWBOBLEManager readAllHistorys];
            break;
        case 101:
            if (self.SWCBPindexMax > 0) {
                if (self.SWCBPindexMax >= self.editCBPindex.text.intValue && self.editCBPindex.text.intValue > 0) {
                    [self.aWBOBLEManager readCBPDataWithIndex:self.editCBPindex.text.intValue Dformat:dformat];
                } else {
                    log = [log stringByAppendingFormat:@"\nThe number of errors is optional."];
                }
            }
            else {
                log = [log stringByAppendingFormat:@"\nPlease read all history data first!"];
            }
            break;
        case 103:
            [self.aWBOBLEManager clearAllHistorys];
            break;
        case 104:
            [self.aWBOBLEManager disconnect];
            break;
        case 105:
            [self.aWBOBLEManager readUserAndVersionData];
            break;
        case 106:
            [self.aWBOBLEManager writeUserID:self.editUserID.text];
            break;
        case 107:
            [self.aWBOBLEManager readSettingValues];
            break;
        case 108:
            [self editSV];
            if ([aSettingValues checkSettingValues]) {
                [self.aWBOBLEManager writeSettingValues:aSettingValues];
            } else {
                log = [log stringByAppendingFormat:@"\nWrong setting value.\n(1)AutoMeasureNumber=1 or 2 and SW_AVG_no_include_first=true\n(2)AutoMeasureNumber=1 and SW_AFib =true\n(3)SW_CBP=true,AutoMeasureNumber*IntervalTime >= 1800"];
            }
            break;
        case 111:
            [self.aWBOBLEManager readDeviceIDAndInfo];
            break;
        case 112:
            [self.aWBOBLEManager readDeviceTime];
            break;
        case 113:
            [self.aWBOBLEManager writeDeviceTime];
            break;
        case 116:
            [self.aWBOBLEManager readFunctionSettingValue];
            break;
        case 119:
            [self.aWBOBLEManager readBTModuleName];
            break;
        case 137:
            [self.aWBOBLEManager startRemoteMeasurementWhtihCBPFunction:dformat];
            break;
        case 138:
            [self.aWBOBLEManager stopRemoteMeasurement];
            break;
        default:
            break;
    }
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.result];
}

#pragma mark - WBODataResponseDelegate
- (void)WBOBLEManagerCellPhoneBluetoothDidUpdateState:(MicroLifeBLEState)state {
    if (state == MicroLifeBLEStatePoweredOn) {
        [self addLogWhitText:@"WBOBLEManagerCellPhoneBluetoothDidUpdateState:MicroLifeBLEStatePoweredOn"];
    }else {
        [self addLogWhitText:@"WBOBLEManagerCellPhoneBluetoothDidUpdateState:MicroLifeBLEStatePoweredOff"];
    }
}

- (void)WBOBLEManagerDidConnectDevice {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerDidConnectDevice:%@",self.aWBOBLEManager.bindingDevice.dictionaryFromModel];
    [self bindingDeviceWhitMacAddres:self.aWBOBLEManager.bindingDevice.macAddress];
    [self addLogWhitText:log];
}

- (void)WBOBLEManagerDidDisconnectDevice {
    [self addLogWhitText:@"WBOLEManagerDidDisconnectDevice"];
    [self.aWBOBLEManager reScan];
}

- (void)WBOBLEManagerDidDiscoverBluetoothDeviceMacAddress:(nonnull NSData *)macAddress Name:(nonnull NSString *)name RSSI:(nonnull NSNumber *)RSSI {
    self.wboBindingDevice = macAddress;
    [self bindingDeviceWhitMacAddres:self.wboBindingDevice];
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerDidDiscoverBluetoothDeviceMacAddress:%@ Name:%@",macAddress,name];
    [self addLogWhitText:log];
}

- (void)WBOBLEManagerDidFailToConnectDevice {
    [self addLogWhitText:@"WBOBLEManagerDidFailToConnectDevice"];
}

- (void)WBOBLEManagerResponseReadUserAndVersionData:(nonnull MicroLifeUserInfo *)user VersionData:(nonnull MicroLifeDeviceInfo *)verData {
    self.editUserID.text = user.BPMuserID;
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseReadUserAndVersionData:%@ VersionData:%@",user.dictionaryFromModel,verData.dictionaryFromModel];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseWriteUserID:(BOOL)isSuccess {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseWriteUserID:%@",isSuccess?@"YES":@"NO"];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseReadDeviceTime:(nonnull MicroLifeDeviceInfo *)deviceInfo {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseReadDeviceTime:%@",deviceInfo.dictionaryFromModel];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseWriteDeviceTime:(BOOL)isSuccess {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseWriteDeviceTime:%@",isSuccess?@"YES":@"NO"];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseWriteSettingValues:(BOOL)isSuccess {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseWriteSettingValues:%@",isSuccess?@"YES":@"NO"];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseReadSettingValue:(nonnull MicroLifeSettingValues *)settingValues {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseReadSettingValue:%@",settingValues.dictionaryFromModel];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
    aSettingValues = settingValues;
    [self setSV];
}

- (void)WBOBLEManagerResponseReadDeviceIDAndInfo:(MicroLifeDeviceInfo *)deviceInfo {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseReadDeviceIDAndInfo:%@",deviceInfo.dictionaryFromModel];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseReadFunctionSettingValue:(MicroLifeFunctionSettingValues *)deviceInfo {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseReadFunctionSettingValue:%@",deviceInfo.dictionaryFromModel];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseReadBTModuleName:(NSString *)BTModuleName {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseReadBTModuleName:%@",BTModuleName];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseStartRemoteMeasurement:(Dformat)D_format {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseStartRemoteMeasurement:%ld",D_format];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseremoteMeasurementStatusEvery5secondsWithSTATUS:(STATUS)status MeasurementNumber:(int)measurementNumber TotalMeasurementNumber:(int)totalMeasurementNumber Countdown:(int)countdown TotalMeasuretime:(int)totalMeasuretime {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseremoteMeasurementStatusEvery5secondsWithSTATUS:%ld MeasurementNumber:%d TotalMeasurementNumber:%d Countdown:%d TotalMeasuretime:%d",status,measurementNumber,totalMeasurementNumber,countdown,totalMeasuretime];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseMeasurementResultsForEachMeasurement:(MicroLifeCurrentAndMData *)data HistoryMeasurementNumber:(int)historyMeasuremeNumber CurrentMeasurementTimes:(int)currentMeasurementTimes AverageCalculationWhenMeasurement:(BOOL)isAverage {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseMeasurementResultsForEachMeasurement:%@ HistoryMeasurementNumber:%d CurrentMeasurementTimes:%d AverageCalculationWhenMeasurement:%@",data.dictionaryFromModel,historyMeasuremeNumber,currentMeasurementTimes,isAverage?@"YES":@"NO"];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseClearHistory:(BOOL)isSuccess {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseClearHistory:%@",isSuccess?@"YES":@"NO"];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseReadAllHistorys:(nonnull MicroLifeDRecord *)data {
    self.SWCBPindexMax = (int)data.MData.count;
    self.editCBPindex.text = [NSString stringWithFormat:@"%d",self.SWCBPindexMax];
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseReadAllHistorys:%@",data.dictionaryFromModel];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

- (void)WBOBLEManagerResponseReadCBPData:(nonnull MicroLifeCBPdataAndCalCBP *)data IsNoData:(BOOL)isNoData {
    NSString *log = [NSString stringWithFormat:@"WBOBLEManagerResponseReadCBPData:%@",isNoData?@"No Data":data.dictionaryFromModel];
    [self addLogWhitText:log];
    [self addLogWhitText:self.aWBOBLEManager.message];
}

@end
